require 'active_support/core_ext/string/inflections'
require 'ostruct'

module Dag
  class Cluster < Model
    include Dag::Client::ClusterValidation

    def initialize(api, cluster)
      super(api)

      @name = cluster['name']
      @status = cluster['status']
      @cluster_status = @status
      @type = cluster['type']
      @instances = cluster['instances']
      @debug = cluster['debug']
    end

    attr_reader :name, :status, :type, :debug

    # restart cluster
    #
    # ==== Parameters
    # * <tt>:force</tt> - restart forcely. false by default
    # * <tt>:type</tt> - cluster type 'DAG5-Hive-Hadoop'
    def restart(force: false, type: nil)
      unless cluster_restart_status?
        raise Dag::Client::StatusInvalid.new("cluster status is invalid: #{cluster_status}")
      end

      params = {
        force: force,
        type: type || @type,
        debug: false
      }

      @api.cluster_restart(@name, params)
    end

    def export_log(params = {})
      unless cluster_norm_or_ptfailed?
        raise Dag::Client::StatusInvalid.new("cluster status is invalid: #{cluster_status}")
      end

      default = {
        compress: false
      }

      @api.cluster_export_log(@name, default.merge(params))
    end

    # return instance infomation
    #
    # == instance detail
    # * <tt>grade</tt>
    # * <tt>quantity</tt>
    def instances
      @instances.map { |instance| os = OpenStruct.new(instance); os.freeze; os }
    end

    # return statistics information
    #
    # == instance detail ==
    # * <tt>instances</tt>
    #   * <tt>instance_id</tt>
    #   * <tt>grade</tt>
    #   * <tt>disk</tt>
    #     * <tt>dfs_used</tt>
    #     * <tt>non_dfs_used</tt>
    #     * <tt>capacity</tt>
    # * <tt>disk</tt>
    #   * <tt>capacity</tt>
    #   * <tt>used</tt>
    def statistics
      statistics_info = if valid_cluster_status? && cluster_norm_or_ptfailed?
                          @api.statistics(@name)
                        end

      if statistics_info.present?
        statistics_info = {
            "instances" => statistics_info['instances'],
            "disk" => statistics_info['disk']
        }
      end

      return nil unless statistics_info
      key_to_underscore(statistics_info)
    end

    private

    def key_to_underscore(hash)
      OpenStruct.new(hash.each_with_object({}) {|(k, v), new_hash|
        v = v.map {|s| send(__method__, s) } if v.class == Array
        v = send(__method__, v) if v.class == Hash
        new_hash[k.underscore] = v
      }).freeze
    end
  end
end
